@@ -11,7 +11,11 @@
     var dbpath = MongoRunner.dataPath + 'sync_write';
     resetDbpath(dbpath);
 
-    var mongodArgs = {dbpath: dbpath, noCleanData: true, journal: ''};
+    var mongodArgs = {
+        dbpath: dbpath,
+        noCleanData: true,
+        journal: ''
+    };
 
     // Start a mongod.
     var conn = MongoRunner.runMongod(mongodArgs);
