@@ -1,21 +1,25 @@
 // This test is skipped on 32-bit platforms
 
 function setupTest() {
-    var s = new ShardingTest({ shards: 2,
-                               mongos: 1,
-                               other: { rs: true,
-                                        numReplicas: 2,
-                                        chunkSize: 1,
-                                        rsOptions: { oplogSize: 50 },
-                                        enableBalancer: 1 } });
+    var s = new ShardingTest({
+        shards: 2,
+        mongos: 1,
+        other: {
+            rs: true,
+            numReplicas: 2,
+            chunkSize: 1,
+            rsOptions: {oplogSize: 50},
+            enableBalancer: true
+        }
+    });
 
     // Reduce chunk size to split
     var config = s.getDB("config");
     config.settings.save({_id: "chunksize", value: 1});
 
-    assert.commandWorked(s.s0.adminCommand({ enablesharding: "test" }));
+    assert.commandWorked(s.s0.adminCommand({enablesharding: "test"}));
     s.ensurePrimaryShard('test', 'test-rs0');
-    assert.commandWorked(s.s0.adminCommand({ shardcollection: "test.foo", key: { "_id": 1 } }));
+    assert.commandWorked(s.s0.adminCommand({shardcollection: "test.foo", key: {"_id": 1}}));
     return s;
 }
 
@@ -29,17 +33,18 @@ function runTest(s) {
 
     if (db.serverBuildInfo().bits == 32) {
         // Make data ~0.5MB for 32 bit builds
-        for (var i = 0; i < 512; i++) str += "a";
-    }
-    else {
+        for (var i = 0; i < 512; i++)
+            str += "a";
+    } else {
         // Make data ~4MB
-        for (var i = 0; i < 4*1024; i++) str += "a";
+        for (var i = 0; i < 4 * 1024; i++)
+            str += "a";
     }
 
     var bulk = db.foo.initializeUnorderedBulkOp();
-    for (j=0; j<100; j++) {
-        for (i=0; i<512; i++) {
-            bulk.insert({ i: idInc++, val: valInc++, y:str });
+    for (j = 0; j < 100; j++) {
+        for (i = 0; i < 512; i++) {
+            bulk.insert({i: idInc++, val: valInc++, y: str});
         }
     }
     assert.writeOK(bulk.execute());
@@ -56,11 +61,12 @@ function runTest(s) {
         print("Shard 1: " + s.shard1.getCollection(db.foo + "").find().itcount());
 
         for (var i = 0; i < 51200; i++) {
-            if(!db.foo.findOne({ i: i }, { i: 1 })) {
+            if (!db.foo.findOne({i: i}, {i: 1})) {
                 print("Could not find: " + i);
             }
 
-            if(i % 100 == 0) print("Checked " + i);
+            if (i % 100 == 0)
+                print("Checked " + i);
         }
 
         print("PROBABLY WILL ASSERT NOW");
@@ -79,15 +85,19 @@ function runTest(s) {
     s.printChunks();
     s.printChangeLog();
 
-    function map() { emit('count', 1); }
-    function reduce(key, values) { return Array.sum(values); }
+    function map() {
+        emit('count', 1);
+    }
+    function reduce(key, values) {
+        return Array.sum(values);
+    }
 
     jsTest.log("Test basic mapreduce...");
 
     // Test basic mapReduce
     for (var iter = 0; iter < 5; iter++) {
         print("Test #" + iter);
-        out = db.foo.mapReduce(map, reduce,"big_out");
+        out = db.foo.mapReduce(map, reduce, "big_out");
     }
 
     print("Testing output to different db...");
@@ -104,7 +114,7 @@ function runTest(s) {
 
         print("Testing mr replace into DB " + iter);
 
-        res = db.foo.mapReduce(map , reduce , { out: { replace: outCollStr, db: outDbStr } });
+        res = db.foo.mapReduce(map, reduce, {out: {replace: outCollStr, db: outDbStr}});
         printjson(res);
 
         outDb = s.getDB(outDbStr);
@@ -112,7 +122,7 @@ function runTest(s) {
 
         obj = outColl.convertToSingleObject("value");
 
-        assert.eq(51200 , obj.count , "Received wrong result " + obj.count);
+        assert.eq(51200, obj.count, "Received wrong result " + obj.count);
 
         print("checking result field");
         assert.eq(res.result.collection, outCollStr, "Wrong collection " + res.result.collection);
@@ -123,81 +133,85 @@ function runTest(s) {
 
     // check nonAtomic output
     assert.throws(function() {
-        db.foo.mapReduce(map, reduce, { out: {replace: "big_out", nonAtomic: true } });
+        db.foo.mapReduce(map, reduce, {out: {replace: "big_out", nonAtomic: true}});
     });
 
     jsTest.log();
 
     // Add docs with dup "i"
     valInc = 0;
-    for (j=0; j<100; j++) {
+    for (j = 0; j < 100; j++) {
         print("Inserted document: " + (j * 100));
         bulk = db.foo.initializeUnorderedBulkOp();
-        for (i=0; i<512; i++) {
-            bulk.insert({ i: idInc++, val: valInc++, y: str });
+        for (i = 0; i < 512; i++) {
+            bulk.insert({i: idInc++, val: valInc++, y: str});
         }
         // wait for replication to catch up
-        assert.writeOK(bulk.execute({ w: 2 }));
+        assert.writeOK(bulk.execute({w: 2}));
     }
 
     jsTest.log("No errors...");
 
-    map2 = function() { emit(this.val, 1); };
-    reduce2 = function(key, values) { return Array.sum(values); };
+    map2 = function() {
+        emit(this.val, 1);
+    };
+    reduce2 = function(key, values) {
+        return Array.sum(values);
+    };
 
     // Test merge
     outcol = "big_out_merge";
 
     // M/R quarter of the docs
     jsTestLog("Test A");
-    out = db.foo.mapReduce(map2, reduce2, { query: {i: {$lt: 25600} }, out: { merge: outcol } });
+    out = db.foo.mapReduce(map2, reduce2, {query: {i: {$lt: 25600}}, out: {merge: outcol}});
     printjson(out);
-    assert.eq(25600 , out.counts.emit , "Received wrong result");
-    assert.eq(25600 , out.counts.output , "Received wrong result");
+    assert.eq(25600, out.counts.emit, "Received wrong result");
+    assert.eq(25600, out.counts.output, "Received wrong result");
 
     // M/R further docs
     jsTestLog("Test B");
     out = db.foo.mapReduce(
-        map2, reduce2, { query: {i: {$gte: 25600, $lt: 51200} }, out: { merge: outcol } });
+        map2, reduce2, {query: {i: {$gte: 25600, $lt: 51200}}, out: {merge: outcol}});
     printjson(out);
-    assert.eq(25600 , out.counts.emit , "Received wrong result");
-    assert.eq(51200 , out.counts.output , "Received wrong result");
+    assert.eq(25600, out.counts.emit, "Received wrong result");
+    assert.eq(51200, out.counts.output, "Received wrong result");
 
     // M/R do 2nd half of docs
     jsTestLog("Test C");
     out = db.foo.mapReduce(
-        map2, reduce2, { query: {i: {$gte: 51200} }, out: { merge: outcol, nonAtomic: true } });
+        map2, reduce2, {query: {i: {$gte: 51200}}, out: {merge: outcol, nonAtomic: true}});
     printjson(out);
-    assert.eq(51200 , out.counts.emit , "Received wrong result");
-    assert.eq(51200 , out.counts.output , "Received wrong result");
-    assert.eq(1 , db[outcol].findOne().value , "Received wrong result");
+    assert.eq(51200, out.counts.emit, "Received wrong result");
+    assert.eq(51200, out.counts.output, "Received wrong result");
+    assert.eq(1, db[outcol].findOne().value, "Received wrong result");
 
     // Test reduce
     jsTestLog("Test D");
     outcol = "big_out_reduce";
 
     // M/R quarter of the docs
-    out = db.foo.mapReduce(map2, reduce2,{ query: { i: { $lt: 25600 } }, out: { reduce: outcol } });
+    out = db.foo.mapReduce(map2, reduce2, {query: {i: {$lt: 25600}}, out: {reduce: outcol}});
     printjson(out);
-    assert.eq(25600 , out.counts.emit , "Received wrong result");
-    assert.eq(25600 , out.counts.output , "Received wrong result");
+    assert.eq(25600, out.counts.emit, "Received wrong result");
+    assert.eq(25600, out.counts.output, "Received wrong result");
 
     // M/R further docs
     jsTestLog("Test E");
     out = db.foo.mapReduce(
-        map2, reduce2, { query: { i: { $gte: 25600, $lt: 51200 } }, out: { reduce: outcol } });
+        map2, reduce2, {query: {i: {$gte: 25600, $lt: 51200}}, out: {reduce: outcol}});
     printjson(out);
-    assert.eq(25600 , out.counts.emit , "Received wrong result");
-    assert.eq(51200 , out.counts.output , "Received wrong result");
+    assert.eq(25600, out.counts.emit, "Received wrong result");
+    assert.eq(51200, out.counts.output, "Received wrong result");
 
     // M/R do 2nd half of docs
     jsTestLog("Test F");
     out = db.foo.mapReduce(
-        map2, reduce2, { query: { i: {$gte: 51200} }, out: { reduce: outcol, nonAtomic: true } });
+        map2, reduce2, {query: {i: {$gte: 51200}}, out: {reduce: outcol, nonAtomic: true}});
     printjson(out);
-    assert.eq(51200 , out.counts.emit , "Received wrong result");
-    assert.eq(51200 , out.counts.output , "Received wrong result");
-    assert.eq(2 , db[outcol].findOne().value , "Received wrong result");
+    assert.eq(51200, out.counts.emit, "Received wrong result");
+    assert.eq(51200, out.counts.output, "Received wrong result");
+    assert.eq(2, db[outcol].findOne().value, "Received wrong result");
 
     // Verify that data is also on secondary
     jsTestLog("Test G");
@@ -208,9 +222,9 @@ function runTest(s) {
     // that replication can keep up even on slow machines.
     s.stopBalancer();
     s._rs[0].test.awaitReplication(300 * 1000);
-    assert.eq(51200 , primary.getDB("test")[outcol].count() , "Wrong count");
+    assert.eq(51200, primary.getDB("test")[outcol].count(), "Wrong count");
     for (var i = 0; i < secondaries.length; ++i) {
-        assert.eq(51200 , secondaries[i].getDB("test")[outcol].count() , "Wrong count");
+        assert.eq(51200, secondaries[i].getDB("test")[outcol].count(), "Wrong count");
     }
 }
 
@@ -218,8 +232,7 @@ var s = setupTest();
 
 if (s.getDB("admin").runCommand("buildInfo").bits < 64) {
     print("Skipping test on 32-bit platforms");
-}
-else {
+} else {
     runTest(s);
 }
 
