@@ -133,6 +133,17 @@ IndexCatalogEntry* IndexCatalog::_setupInMemoryStructures(OperationContext* txn,
 
     Status status = _isSpecOk(descriptor->infoObj());
     if (!status.isOK() && status != ErrorCodes::IndexAlreadyExists) {
+        if (_collection->ns().ns() == "admin.system.version") {
+            auto indexNameElem = descriptor->infoObj()["name"];
+
+            if (indexNameElem.type() == BSONType::String &&
+                indexNameElem.valueStringData() == StringData("incompatible_with_version_32")) {
+                severe()
+                    << "Cannot start mongod when the featureCompatibilityVersion is higher than "
+                       "3.2. See http://dochub.mongodb.org/core/3.4-feature-compatibility.";
+                fassertFailedNoTrace(40352);
+            }
+        }
         severe() << "Found an invalid index " << descriptor->infoObj() << " on the "
                  << _collection->ns().ns() << " collection: " << status.reason();
         fassertFailedNoTrace(28782);
@@ -365,6 +376,11 @@ Status IndexCatalog::IndexBuildBlock::init() {
     const bool initFromDisk = false;
     _entry = _catalog->_setupInMemoryStructures(_txn, descriptorCleaner.release(), initFromDisk);
 
+    // Register this index with the CollectionInfoCache to regenerate the cache. This way, updates
+    // occurring while an index is being build in the background will be aware of whether or not
+    // they need to modify any indexes.
+    _collection->infoCache()->addedIndex(_txn, descriptor);
+
     return Status::OK();
 }
 
@@ -413,8 +429,6 @@ void IndexCatalog::IndexBuildBlock::success() {
     });
 
     entry->setIsReady(true);
-
-    collection->infoCache()->addedIndex(_txn, desc);
 }
 
 namespace {
