@@ -170,6 +170,14 @@ private:
     void _producerThread();
     void _produce(OperationContext* txn);
 
+    /**
+     * Signals to the applier that we have no new data,
+     * and are in sync with the applier at this point.
+     *
+     * NOTE: Used after rollback and during draining to transition to Primary role;
+     */
+    void _signalNoNewDataForApplier();
+
     /**
      * Processes query responses from fetcher.
      */
@@ -179,7 +187,8 @@ private:
                           OpTime lastOpTimeFetched,
                           long long lastFetchedHash,
                           Milliseconds fetcherMaxTimeMS,
-                          Status* returnStatus);
+                          Status* returnStatus,
+                          int rbid);
 
     /**
      * Executes a rollback.
@@ -187,6 +196,7 @@ private:
      */
     void _rollback(OperationContext* txn,
                    const HostAndPort& source,
+                   boost::optional<int> requiredRBID,
                    stdx::function<DBClientBase*()> getConnection);
 
     /**
