@@ -225,7 +225,14 @@ WiredTigerIndex::WiredTigerIndex(OperationContext* ctx,
     Status versionStatus = WiredTigerUtil::checkApplicationMetadataFormatVersion(
         ctx, uri, kMinimumIndexVersion, kMaximumIndexVersion);
     if (!versionStatus.isOK()) {
-        fassertFailedWithStatusNoTrace(28579, versionStatus);
+        str::stream ss;
+        ss << versionStatus.reason() << " Index: {name: " << desc->indexName()
+           << ", ns: " << desc->parentNS() << "} - version too new for this mongod."
+           << " See http://dochub.mongodb.org/core/3.4-index-downgrade for detailed"
+           << " instructions on how to handle this error.";
+        Status indexVersionStatus(
+            ErrorCodes::UnsupportedFormat, ss.ss.str(), versionStatus.location());
+        fassertFailedWithStatusNoTrace(28579, indexVersionStatus);
     }
 }
 
@@ -391,7 +398,41 @@ Status WiredTigerIndex::touch(OperationContext* txn) const {
 
 
 long long WiredTigerIndex::getSpaceUsedBytes(OperationContext* txn) const {
-    WiredTigerSession* session = WiredTigerRecoveryUnit::get(txn)->getSession(txn);
+    auto ru = WiredTigerRecoveryUnit::get(txn);
+    WiredTigerSession* session = ru->getSession(txn);
+
+    if (ru->getSessionCache()->isEphemeral()) {
+        // For ephemeral case, use cursor statistics
+        const auto statsUri = "statistics:" + uri();
+
+        // Helper function to retrieve stats and check for errors
+        auto getStats = [&](int key) -> int64_t {
+            StatusWith<int64_t> result = WiredTigerUtil::getStatisticsValueAs<int64_t>(
+                session->getSession(), statsUri, "statistics=(fast)", key);
+            if (!result.isOK()) {
+                if (result.getStatus().code() == ErrorCodes::CursorNotFound)
+                    return 0;  // ident gone, so return 0
+
+                uassertStatusOK(result.getStatus());
+            }
+            return result.getValue();
+        };
+
+        auto inserts = getStats(WT_STAT_DSRC_CURSOR_INSERT);
+        auto removes = getStats(WT_STAT_DSRC_CURSOR_REMOVE);
+        auto insertBytes = getStats(WT_STAT_DSRC_CURSOR_INSERT_BYTES);
+
+        if (inserts == 0 || removes >= inserts)
+            return 0;
+
+        // Rough approximation of index size as average entry size times number of entries.
+        // May be off if key sizes change significantly over the life time of the collection,
+        // but is the best we can do currrently with the statistics available.
+        auto bytesPerEntry = (insertBytes + inserts - 1) / inserts;  // round up
+        auto numEntries = inserts - removes;
+        return numEntries * bytesPerEntry;
+    }
+
     return static_cast<long long>(WiredTigerUtil::getIdentSize(session->getSession(), _uri));
 }
 
@@ -453,9 +494,13 @@ protected:
 
         // Not using cursor cache since we need to set "bulk".
         WT_CURSOR* cursor;
-        // We use our own session to ensure we aren't in a transaction.
+        // Use a different session to ensure we don't hijack an existing transaction.
+        // Configure the bulk cursor open to fail quickly if it would wait on a checkpoint
+        // completing - since checkpoints can take a long time, and waiting can result in
+        // an unexpected pause in building an index.
         WT_SESSION* session = _session->getSession();
-        int err = session->open_cursor(session, idx->uri().c_str(), NULL, "bulk", &cursor);
+        int err = session->open_cursor(
+            session, idx->uri().c_str(), NULL, "bulk,checkpoint_wait=false", &cursor);
         if (!err)
             return cursor;
 
@@ -1045,8 +1090,17 @@ void WiredTigerIndexUnique::_unindex(WT_CURSOR* c,
     // dups are allowed, so we have to deal with a vector of RecordIds.
 
     int ret = WT_OP_CHECK(c->search(c));
-    if (ret == WT_NOTFOUND)
+    if (ret == WT_NOTFOUND) {
+        // WT_NOTFOUND is only expected during a background index build. Insert a dummy value and
+        // delete it again to trigger a write conflict in case this is being concurrently indexed by
+        // the background indexer.
+        c->set_key(c, keyItem.Get());
+        c->set_value(c, emptyItem.Get());
+        invariantWTOK(WT_OP_CHECK(c->insert(c)));
+        c->set_key(c, keyItem.Get());
+        invariantWTOK(WT_OP_CHECK(c->remove(c)));
         return;
+    }
     invariantWTOK(ret);
 
     WT_ITEM old;
@@ -1154,6 +1208,15 @@ void WiredTigerIndexStandard::_unindex(WT_CURSOR* c,
     int ret = WT_OP_CHECK(c->remove(c));
     if (ret != WT_NOTFOUND) {
         invariantWTOK(ret);
+    } else {
+        // WT_NOTFOUND is only expected during a background index build. Insert a dummy value and
+        // delete it again to trigger a write conflict in case this is being concurrently indexed by
+        // the background indexer.
+        c->set_key(c, item.Get());
+        c->set_value(c, emptyItem.Get());
+        invariantWTOK(WT_OP_CHECK(c->insert(c)));
+        c->set_key(c, item.Get());
+        invariantWTOK(WT_OP_CHECK(c->remove(c)));
     }
 }
 
