@@ -421,6 +421,70 @@ var ReplSetTest = function(opts) {
         }, "Awaiting secondaries", timeout);
     };
 
+    /**
+     * Blocks until the specified node says it's syncing from the given upstream node.
+     */
+    this.awaitSyncSource = function(node, upstreamNode, timeout) {
+        print("Waiting for node " + node.name + " to start syncing from " + upstreamNode.name);
+        var status = null;
+        assert.soonNoExcept(
+            function() {
+                status = node.getDB("admin").runCommand({replSetGetStatus: 1});
+                for (var j = 0; j < status.members.length; j++) {
+                    if (status.members[j].self) {
+                        return status.members[j].syncingTo === upstreamNode.host;
+                    }
+                }
+                return false;
+            },
+            "Awaiting node " + node + " syncing from " + upstreamNode + ": " + tojson(status),
+            timeout);
+    };
+
+    /**
+     * Blocks until all nodes agree on who the primary is.
+     */
+    this.awaitNodesAgreeOnPrimary = function(timeout) {
+        timeout = timeout || 60000;
+
+        assert.soon(function() {
+            try {
+                var primary = -1;
+
+                for (var i = 0; i < self.nodes.length; i++) {
+                    var replSetGetStatus =
+                        self.nodes[i].getDB("admin").runCommand({replSetGetStatus: 1});
+                    var nodesPrimary = -1;
+                    for (var j = 0; j < replSetGetStatus.members.length; j++) {
+                        if (replSetGetStatus.members[j].state === ReplSetTest.State.PRIMARY) {
+                            // Node sees two primaries.
+                            if (nodesPrimary !== -1) {
+                                return false;
+                            }
+                            nodesPrimary = j;
+                        }
+                    }
+                    // Node doesn't see a primary.
+                    if (nodesPrimary < 0) {
+                        return false;
+                    }
+
+                    if (primary < 0) {
+                        // If we haven't seen a primary yet, set it to this.
+                        primary = nodesPrimary;
+                    } else if (primary !== nodesPrimary) {
+                        return false;
+                    }
+                }
+
+                return true;
+            } catch (e) {
+                print("caught exception " + e);
+                return false;
+            }
+        }, "Awaiting nodes to agree on primary", timeout);
+    };
+
     /**
      * Blocking call, which will wait for a primary to be elected for some pre-defined timeout and
      * if primary is available will return a connection to it. Otherwise throws an exception.
@@ -561,6 +625,28 @@ var ReplSetTest = function(opts) {
         }
     };
 
+    /**
+     * Blocks until all nodes in the replica set have the same config version as the primary.
+     **/
+    this.awaitNodesAgreeOnConfigVersion = function(timeout) {
+        timeout = timeout || this.kDefaultTimeoutMS;
+
+        assert.soonNoExcept(function() {
+            var primaryVersion = self.getPrimary().adminCommand({ismaster: 1}).setVersion;
+
+            for (var i = 0; i < self.nodes.length; i++) {
+                var version = self.nodes[i].adminCommand({ismaster: 1}).setVersion;
+                assert.eq(version,
+                          primaryVersion,
+                          "waiting for secondary node " + self.nodes[i].host +
+                              " with config version of " + version +
+                              " to match the version of the primary " + primaryVersion);
+            }
+
+            return true;
+        }, "Awaiting nodes to agree on config version", timeout);
+    };
+
     /**
      * Waits for the last oplog entry on the primary to be visible in the committed snapshop view
      * of the oplog on *all* secondaries.
@@ -688,7 +774,7 @@ var ReplSetTest = function(opts) {
                               ", but expected config version #" + configVersion);
 
                         if (slaveConfigVersion > configVersion) {
-                            master = this.getPrimary();
+                            master = self.getPrimary();
                             configVersion =
                                 master.getDB("local")['system.replset'].findOne().version;
                             masterOpTime = _getLastOpTime(master);
@@ -873,6 +959,12 @@ var ReplSetTest = function(opts) {
 
         printjson(this.nodes);
 
+        // Clean up after noReplSet to ensure it doesn't effect future restarts.
+        if (options.noReplSet) {
+            this.nodes[n].fullOptions.replSet = defaults.replSet;
+            delete this.nodes[n].fullOptions.noReplSet;
+        }
+
         wait = wait || false;
         if (!wait.toFixed) {
             if (wait)
